//	GamePlayState.h - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEPLAYSTATE_H_
#define GAMEPLAYSTATE_H_

#include "drawing/TextLayout.h"
#include "ParticleGenerator.h"
#include "GameCounter.h"
#include "c0xHeaders.h"
#include "GameState.h"
#include "Tweener.h"
#include "Star.h"

class MinimalSounder;
class Player;
class Ball;
class NPC;

///	The main gameplay state.
class GamePlayState : public GameState,
					  public CounterNotification
{
  public:
	///	Constructor.
	GamePlayState(Application& app, SounderPtr& sound);
	///	Destructor.
	~GamePlayState();

	///	Called frequently to update the state.
	void update();
	///	Called to draw the state.
	void draw(DrawerPtr& d, float timestepInterp);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);
	///	Called when a key up event is received.
	void keyUp(const std::wstring& action,
			   const std::wstring& keyPressed,
			   int keyModifier);

	///	Called when a joystick axis changes.
	void joyMove(const std::wstring& action,
				 float value,
				 int joystick,
				 int axis);

	///	Called when the related counter reaches the specified value.
	void countReached(const std::wstring& counter, int value);

	///	Helper method. Sets up the screen shake.
	void shakeScreen();

	///	Returns the name of this state.
	const std::wstring getName() const {return L"GamePlayState";};
	///	Used to construct instances of this state.
	static GameStateCreator<GamePlayState> creator;
  private:
	///	Handles the collision detection.
	void collisionDetection();
	///	Helper method. Handles stuff happening when someone scores.
	void handleScore(bool player);
	///	Helper method. Adds a random apple to the game.
	void addApple();
	///	Helper method. Passes existing key presses onto a new player.
	void setupPlayerKeys(int index);

	///	The different player types.
	enum PlayerType
	{
		Pong = 0,
		Snake,
		Football,
		CaptureTheFlag,
		ZombieApocalypse,
		Dead
	};

	///	Used for the ticker text.
	TextLayout tickerText;
	///	Used for the game over text.
	TextLayout gameOverText;
	///	Used for the game over text.
	std::wstring gameOverString;
	///	Used for the game over text.
	ThreeFloats gameOverColour;

	///	Our particle generator.
	ParticleGenerator particles;

	///	Used by the screen shake.
	float shakeVal;
	///	Used by the screen shake.
	float shakeIndex;

	///	Keeps track of the scores.
	GameCounter gameCounter;
	///	True if the players' colours have been set.
	bool coloursSet;

	///	The left player's current type.
	int leftType;
	///	The right player's current type.
	int rightType;

	///	True if the left player now has goals.
	bool leftGoals;
	///	True if the right player now has goals.
	bool rightGoals;

	///	The two players.
	shared_ptr<Player> players[2];
	///	The ball.
	shared_ptr<Ball> ball;
	///	Any NPCs in the game.
	std::vector<shared_ptr<NPC> > npcs;
	///	> 0 if the player is currently firing.
	int firing[2];
	///	The last joystick position for twin-stick firing.
	TwoFloats lastJoy[2];

	///	Capture the flag bases.
	Star flagBases[2];

	///	Used to generate zombies.
	int zombieCount[2];

	///	The background colour.
	ThreeFloats backgroundColour;
	///	Used to modulate the background colour.
	STweener<ThreeFloats> backgroundTween;
	///	The walls colour.
	ThreeFloats wallColour;
	///	Used to modulate the wall colour.
	STweener<ThreeFloats> wallTween;

	///	The stars colour.
	ThreeFloats starColour;
	///	The apples colour.
	ThreeFloats appleColour;

	///	Used with the shader.
	float noiseSeedInc;
	///	Used with the shader.
	int noiseCount;
	///	Used to flash the screen with heavy saturation + contrast.
	float satCon;
	///	Used to amp saturation as the game goes on.
	float saturation;

	///	Player inputs.
	bool upIsDown[2];
	///	Player inputs.
	bool downIsDown[2];
	///	Player inputs.
	bool leftIsDown[2];
	///	Player inputs.
	bool rightIsDown[2];
	///	Player inputs.
	TwoFloats joyLast[2];

	///	Our sounder.
	shared_ptr<MinimalSounder> sounder;
};

#endif
