//	MinimalSounder.cpp - The game's sounder.
//  ----------------------------------------------------------------------------
//	This file is part of 'SoManyJaggedShards', an entry for 7DRL 2013.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "MinimalSounder.h"
#include "FilePath.h"
#include "UTF8File.h"
#include "Random.h"

#include "synth/Collision.inl"
#include "synth/GameOver.inl"
#include "synth/Score.inl"
#include "synth/Shoot.inl"

#include <cassert>
#include <sstream>
#include <string>

//------------------------------------------------------------------------------
MinimalSounder::MinimalSounder():
brushStrokesPitch(0),
distortGain(0.5f),
loud(false),
currentPitchSet(-1),
fadeOut(1.0f)
{
	int i, j;
	FilePath pitchesDir(FilePath::getDataFilesDir().getChild(L"pitches"));

	for(i=0;i<pitchesDir.getNumChildren();++i)
	{
		std::wstring tempstr;
		UTF8File phil(pitchesDir.getChild(i));

		pitchSets.push_back(std::vector<float>());

		while(!phil.readLine(tempstr))
		{
			float tempf;
			std::wstringstream tempstr2;

			tempstr2 << tempstr;
			tempstr2 >> tempf;

			pitchSets.back().push_back(tempf);
		}
	}

	for(j=0;j<2;++j)
	{
		for(i=0;i<BufferSize;++i)
			granBuffers[j][i] = 0.0f;
	}

	pickNewPitches();

	brushStrokes.setActive(true);
	collisionSynth.loadParameters(Collision::numParameters, Collision::parameters);
	gameOverSynth.loadParameters(GameOver::numParameters, GameOver::parameters);
	shootSynth.loadParameters(Shoot::numParameters, Shoot::parameters);
	scoreSynth.loadParameters(Score::numParameters, Score::parameters);
	delay.setDelay(0.66f);
	delay.setFeedback(0.66f);
	delay.setMix(0.33f);
	granulators[0].setDensity(0.66f);
	granulators[0].setDuration(2.0f);
	granulators[0].setPitch(1.0f * powf(2.0f, 5.0f/12.0f));
	granulators[0].setMaxBlockSize(44100*2);
	granulators[1].setDensity(0.66f);
	granulators[1].setDuration(2.0f);
	granulators[1].setPitch(1.0f * powf(2.0f, 7.0f/12.0f));
	granulators[1].setMaxBlockSize(44100*2);
	combFilters.setMix(0.33f);
	combFilters.setOutputLevel(0.05f);
}

//------------------------------------------------------------------------------
MinimalSounder::~MinimalSounder()
{

}

//------------------------------------------------------------------------------
void MinimalSounder::getAudio(float *input, float *output, int numSamples)
{
	int i;
	float dist;
	float tempf;

	assert(numSamples < (BufferSize/2));

	//Just in case.
	for(i=0;i<(numSamples*2);++i)
		output[i] = 0.0f;

	brushStrokes.getAudio(output, numSamples);

	collisionSynth.getAudio(output, numSamples);
	gameOverSynth.getAudio(output, numSamples);
	shootSynth.getAudio(output, numSamples);
	scoreSynth.getAudio(output, numSamples);

	//Process the granulators.
	for(i=0;i<(numSamples*2);++i)
	{
		granBuffers[0][i] = output[i];
		granBuffers[1][i] = output[i];
	}
	granulators[0].processBlock(granBuffers[0], numSamples);
	granulators[1].processBlock(granBuffers[1], numSamples);
	for(i=0;i<(numSamples*2);++i)
	{
		output[i] += granBuffers[0][i];
		output[i] += granBuffers[1][i];
	}

	delay.getAudio(output, numSamples);
	combFilters.processBlock(output, numSamples);

	if(loud)
	{
		tempf = 1.0f - (fakeLog(distortGain) * 0.9f);
		dist = 1.0f+(fakeLog(distortGain)*9.0f);
		for(i=0;i<(numSamples*2);++i)
			output[i] = tanhf(output[i]*dist) * tempf;
	}

	if(fadeOut < 1.0f)
	{
		for(i=0;i<numSamples;++i)
		{
			output[(i*2)] *= fadeOut;
			output[(i*2)+1] *= fadeOut;

			if(fadeOut > 0.0f)
				fadeOut -= 0.00001f;
		}
	}
}

//------------------------------------------------------------------------------
void MinimalSounder::reset()
{
	TwoFloats emptyTrail[BrushStrokesSynth::TableSize];

	brushStrokes.setTables(emptyTrail);
}

//------------------------------------------------------------------------------
void MinimalSounder::triggerFadeOut()
{
	fadeOut -= 0.00001f;
}

//------------------------------------------------------------------------------
void MinimalSounder::setBrushStrokesPitch(int val)
{
	assert(val > -1);
	assert(val < (int)pitchSets[0].size());

	brushStrokes.setFrequency(pitchSets[currentPitchSet][val] * 0.25f);
}

//------------------------------------------------------------------------------
void MinimalSounder::setBrushStrokesTable(const TwoFloats *table)
{
	brushStrokes.setTables(table);
}

//------------------------------------------------------------------------------
void MinimalSounder::triggerCollision(int val)
{
	collisionSynth.triggerNote(pitchSets[currentPitchSet][val] * 0.25f);
}

//------------------------------------------------------------------------------
void MinimalSounder::triggerGameOver(int val)
{
	gameOverSynth.triggerNote(pitchSets[currentPitchSet][val] * 0.25f);
}

//------------------------------------------------------------------------------
void MinimalSounder::triggerScore(int val)
{
	scoreSynth.triggerNote(pitchSets[currentPitchSet][val] * 0.25f);
}

//------------------------------------------------------------------------------
void MinimalSounder::triggerShoot(int val)
{
	shootSynth.triggerNote(pitchSets[currentPitchSet][val] * 0.25f);
}

//------------------------------------------------------------------------------
void MinimalSounder::incDistortGain()
{
	if(distortGain < 0.5f)
		distortGain += 0.01f;
}

//------------------------------------------------------------------------------
void MinimalSounder::setLoud(bool val)
{
	loud = val;
}

//------------------------------------------------------------------------------
void MinimalSounder::pickNewPitches()
{
	int i;
	int tempint = Random::getGlobal().getRandomInt(pitchSets.size());

	while(tempint == currentPitchSet)
	{
		++tempint;
		tempint %= pitchSets.size();
	}
	currentPitchSet = tempint;

	brushStrokes.setFrequency(pitchSets[currentPitchSet][brushStrokesPitch]);

	for(i=0;i<8;++i)
		combFilters.setPitch(i, pitchSets[currentPitchSet][i]);
}
