//	CombFilterBank.cpp - Bank of 8 tuned comb filters.
//	----------------------------------------------------------------------------
//	This file is part of NiallGran, a VST plugin.
//	Copyright (c) 2013 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "CombFilterBank.h"

#include <cassert>

//------------------------------------------------------------------------------
CombFilterBank::CombFilterBank():
outputLevel(0.15f),
mix(1.0f)
{
	int i, j;

	dcSpeed[0] = 0.0f;
	dcSpeed[1] = 0.0f;
	dcPos[0] = 0.0f;
	dcPos[1] = 0.0f;

	dcIn[0] = 0.0f;
	dcIn[1] = 0.0f;
	dcOut[0] = 0.0f;
	dcOut[1] = 0.0f;
	dcCoeff = 1.0f-(126.0f/44100.0f);

	for(i=0;i<NumFilters;++i)
	{
		for(j=0;j<MaxDelayLength;++j)
		{
			filters[i].delayLine[0][j] = 0.0f;
			filters[i].delayLine[1][j] = 0.0f;
		}

		filters[i].frequency = 440.0f;
		filters[i].length = 44100.0f/filters[i].frequency;
		filters[i].newLength = 44100.0f/filters[i].frequency;
		filters[i].glissando = 10000.0f;
		filters[i].readPos = 0.0f;
		filters[i].writePos = filters[i].length;
		filters[i].feedback = 0.999f;
		filters[i].level = 1.0f;
	}

	for(i=0;i<MaxDelayLength;++i)
	{
		tempBuffer[0][i] = 0.0f;
		tempBuffer[1][i] = 0.0f;
	}
}

//------------------------------------------------------------------------------
CombFilterBank::~CombFilterBank()
{

}

//------------------------------------------------------------------------------
void CombFilterBank::processBlock(float *buffer, int numSamples)
{
	int i, j;
	int left;
	int right;
	int index;
	float fraction;
	float sampleVal;
	const float BufferSize = (float)MaxDelayLength;

	//Clear the temp buffer.
	for(i=0;i<numSamples;++i)
	{
		tempBuffer[0][i] = 0.0f;
		tempBuffer[1][i] = 0.0f;
	}

	//Get the filters' audio.
	for(j=0;j<NumFilters;++j)
	{
		for(i=0;i<numSamples;++i)
		{
			index = (int)filters[j].writePos;
			left = (int)filters[j].readPos;
			right = left + 1;
			right %= MaxDelayLength;
			fraction = filters[j].readPos - (float)left;

			//Left channel.
			{
				filters[j].delayLine[0][index] = buffer[i*2];

				sampleVal = filters[j].delayLine[0][left] * (1.0f-fraction);
				sampleVal += filters[j].delayLine[0][right] * fraction;

				filters[j].delayLine[0][index] += sampleVal * filters[j].feedback;
				filters[j].delayLine[0][index] += 1e-18f;
				filters[j].delayLine[0][index] -= 1e-18f;

				tempBuffer[0][i] += (buffer[i*2] + sampleVal) * filters[j].level;
			}

			//Right channel.
			{
				filters[j].delayLine[1][index] = buffer[(i*2)+1];

				sampleVal = filters[j].delayLine[1][left] * (1.0f-fraction);
				sampleVal += filters[j].delayLine[1][right] * fraction;

				filters[j].delayLine[1][index] += sampleVal * filters[j].feedback;
				filters[j].delayLine[1][index] += 1e-18f;
				filters[j].delayLine[1][index] -= 1e-18f;

				tempBuffer[1][i] += (buffer[(i*2)+1] + sampleVal) * filters[j].level;
			}

			filters[j].writePos += 1.0f;
			if(filters[j].writePos >= BufferSize)
				filters[j].writePos = 0.0f;

			if(filters[j].length != filters[j].newLength)
			{
				filters[j].length += filters[j].lengthInc;
				if((filters[j].lengthInc > 0.0f) && (filters[j].length >= filters[j].newLength))
					filters[j].length = filters[j].newLength;
				else if((filters[j].lengthInc < 0.0f) && (filters[j].length <= filters[j].newLength))
					filters[j].length = filters[j].newLength;
			}

			filters[j].readPos = filters[j].writePos - filters[j].length;
			if(filters[j].readPos < 0.0f)
				filters[j].readPos += BufferSize;
		}
	}

	//Write tempBuffer to output.
	for(i=0;i<numSamples;++i)
	{
		tempBuffer[0][i] *= mix * outputLevel;
		tempBuffer[0][i] += buffer[(i*2)] * (1.0f-mix);
		buffer[(i*2)] = tempBuffer[0][i] - dcIn[0] + dcCoeff*(dcOut[0]);
		dcIn[0] = tempBuffer[0][i];
		dcOut[0] = buffer[(i*2)];
		
		tempBuffer[1][i] *= mix * outputLevel;
		tempBuffer[1][i] += buffer[(i*2)+1] * (1.0f-mix);
		buffer[(i*2)+1] = tempBuffer[1][i] - dcIn[1] + dcCoeff*(dcOut[1]);
		dcIn[1] = tempBuffer[1][i];
		dcOut[1] = buffer[(i*2)+1];
	}
}

//------------------------------------------------------------------------------
void CombFilterBank::setPitch(int index, float val)
{
	filters[index].frequency = val;
	filters[index].newLength = 44100.0f/filters[index].frequency;
	filters[index].lengthInc = ((filters[index].newLength-filters[index].length)/filters[index].glissando);
}

//------------------------------------------------------------------------------
void CombFilterBank::setFeedback(int index, float val)
{
	filters[index].feedback = (val * 0.0999f) + 0.9f;
}

//------------------------------------------------------------------------------
void CombFilterBank::setLevel(int index, float val)
{
	filters[index].level = val;
}

//------------------------------------------------------------------------------
void CombFilterBank::setOutputLevel(float val)
{
	outputLevel = val;
}

//------------------------------------------------------------------------------
void CombFilterBank::setMix(float val)
{
	mix = val;
}
