//	CombFilterBank.h - Bank of 8 tuned comb filters.
//	----------------------------------------------------------------------------
//	This file is part of NiallGran, a VST plugin.
//	Copyright (c) 2013 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef COMBFILTERBANK_H_
#define COMBFILTERBANK_H_

#include "c0xHeaders.h"

///	Bank of 8 tuned comb filters.
class CombFilterBank
{
  public:
	///	Constructor.
	CombFilterBank();
	///	Destructor.
	~CombFilterBank();

	///	Processes the input audio, writes the granulated output to the passed-in buffer.
	/*!
		\param buffer Contains the input audio, which should be overwritten
		with the plugin's output. NOTE: this is an INTERLEAVED stereo array.
		\param numSamples The number of samples in this buffer.

		Assumes a stereo input/output.
	 */
	void processBlock(float *buffer, int numSamples);

	///	Sets the pitch of the indexed filter.
	void setPitch(int index, float val);
	///	Sets the feedback of the indexed filter.
	void setFeedback(int index, float val);
	///	Sets the level of the indexed filter.
	void setLevel(int index, float val);

	///	Sets the global output level.
	void setOutputLevel(float val);
	///	Sets the mix value (0->1).
	void setMix(float val);
  private:
	///	Some constants.
	enum
	{
		MaxDelayLength = 88200,
		NumFilters = 8
	};
	///	Struct representing a single filter.
	struct Filter
	{
		/// The two delay lines.
		float delayLine[2][MaxDelayLength];

		///	The current write position within the delay line.
		float writePos;
		///	The current read position within the delay line.
		float readPos;

		///	The filter's frequency.
		float frequency;
		///	The length of the delay.
		float length;

		///	Used to glissando to a new frequency.
		float lengthInc;
		///	Used to glissando to a new frequency.
		float newLength;
		///	Used to set the glissando time.
		float glissando;

		///	The filter's feedback.
		float feedback;
		///	The filter's level.
		float level;
	};

	///	The 8 filters.
	Filter filters[NumFilters];

	///	Temporary buffer.
	float tempBuffer[2][MaxDelayLength];

	///	The output level.
	float outputLevel;
	///	The mix value.
	float mix;

	///	For the dc filter on the output.
	float dcSpeed[2];
	///	For the dc filter on the output.
	float dcPos[2];

	///	Temp.
	float dcIn[2];
	///	Temp.
	float dcOut[2];
	///	Temp.
	float dcCoeff;
};

#endif
